package ci;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.geom.GeneralPath;
import java.awt.geom.Line2D;

public class DensityView extends DrawingComponent implements IScale
{
	private IDensity m_dens = new ExponentialDensity(5);
	
	private double m_dXScale =  10.0; 
	private double m_dYScale = -1000.0; 
	
	
	@Override
	protected void paintComponent(Graphics g)
	{
		super.paintComponent(g);
		
        drawAxes(g);

        m_dens.draw(g, this);
	}
	

	protected void drawAxes(Graphics g)
	{
        if (g == null)
            return;
        
        Graphics2D g2 = (Graphics2D)g.create();
        
        try
        {
        	float xmax = (float)((m_dens.getXmax()*1.1)*m_dXScale); 
        	float ymax = (float)((m_dens.getYmax()*1.1)*m_dYScale);
        	
            GeneralPath shpArrowRight = new GeneralPath();
            shpArrowRight.moveTo(xmax, 0);
            shpArrowRight.lineTo(xmax, 5);
            shpArrowRight.lineTo(xmax+15f, 0);
            shpArrowRight.lineTo(xmax, -5);
            shpArrowRight.lineTo(xmax, 0);

            GeneralPath shpArrowUp = new GeneralPath();
            shpArrowUp.moveTo(0f,  ymax);
            shpArrowUp.lineTo(5f,  ymax);
            shpArrowUp.lineTo(0f,  ymax-15);
            shpArrowUp.lineTo(-5,  ymax);
            shpArrowUp.lineTo(0f,  ymax);
            
            
            Line2D.Double lineXAxis = new Line2D.Double(0, 0, xmax, 0);
            Line2D.Double lineYAxis = new Line2D.Double(0, 0f, 0, ymax);
            g2.setColor(new Color(0, 0, 0));
            g2.draw(lineXAxis);
            g2.draw(lineYAxis);
            g2.fill(shpArrowRight);
            g2.fill(shpArrowUp);

            

        } finally
        {
            g2.dispose();
        }
	}


	public double getXScale()
	{
		return m_dXScale;
	}


	public double getYScale()
	{
		return m_dYScale;
	}
	
	public int sample(int nSize, int nCount, double alpha)
	{
		int good_ci = m_dens.sampleAndCalcCI(nSize, nCount, alpha);
		repaint();
		
		return good_ci;
	}


	public double getYScaleAbs()
	{
		if (m_dYScale < 0)
			return -m_dYScale;
		else
			return m_dYScale;
	}
	
	public void setTheta(double theta)
	{
		m_dens.setParameter(theta);
		repaint();
	}
}
