package ci;

import java.awt.Image;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.util.Locale;

import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JSlider;
import javax.swing.JTextField;
import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import ci.res.ResRoot;

import com.jeta.forms.components.panel.FormPanel;
import com.jeta.forms.gui.form.FormAccessor;
import com.jgoodies.looks.plastic.PlasticXPLookAndFeel;

public class MainFrame extends JFrame
{
	private FormPanel   m_pnlMain = null;
	private DensityView m_view = null;
	
	private double      m_alpha = 0.05;
	private JSlider     m_sldAlpha = null;
	private JTextField  m_txtAlpha = null;

	private JSlider     m_sldSampleSize = null;
	private JTextField  m_txtSampleSize = null;

	private JSlider     m_sldSampleCount = null;
	private JTextField  m_txtSampleCount = null;

	private JSlider     m_sldTheta = null;
	private JTextField  m_txtTheta = null;
	
	private JLabel      m_lblRatio = null;
	private JLabel      m_lblEquation = null;
	
	public MainFrame()
	{
		this.setSize(900, 700);
		this.setTitle("Confidence Interval Examples");
		this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		
		
		try
		{
			m_pnlMain = new FormPanel(ResRoot.class.getResourceAsStream("dlg-main.xml"));
			
		} catch (Exception ex)
		{
			System.err.println(ex);
			System.exit(1);
		}
		
		this.getContentPane().add(m_pnlMain);
	
		m_view = new DensityView();
		FormAccessor accessor = m_pnlMain.getFormAccessor();
		accessor.replaceBean("lblPlaceholder", m_view);
		
		
		m_sldAlpha       = (JSlider)m_pnlMain.getFormAccessor("grdSettings").getComponentByName("sldAlpha");
		m_sldSampleSize  = (JSlider)m_pnlMain.getFormAccessor("grdSettings").getComponentByName("sldSampleSize");
		m_sldSampleCount = (JSlider)m_pnlMain.getFormAccessor("grdSettings").getComponentByName("sldSampleCount");
		m_sldTheta       = (JSlider)m_pnlMain.getFormAccessor("grdSettings").getComponentByName("sldTheta");

		m_txtAlpha       = m_pnlMain.getTextField("txtAlpha");
		m_txtSampleSize  = m_pnlMain.getTextField("txtSampleSize");
		m_txtSampleCount = m_pnlMain.getTextField("txtSampleCount");
		m_txtTheta       = m_pnlMain.getTextField("txtTheta");
		
		
		m_sldAlpha.addChangeListener(
		    new ChangeListener()
		    {
				public void stateChanged(ChangeEvent e)
				{
					m_alpha = 100 - m_sldAlpha.getValue();
					m_txtAlpha.setText(String.format(Locale.US, "%.4f", 100-m_alpha));
				}
		    });
		
		m_txtAlpha.addKeyListener(
			new KeyAdapter()
			{
				public void keyPressed(KeyEvent ev)
				{
					if (ev.getKeyCode() == KeyEvent.VK_ENTER)
					{
						double new_alpha = 0;
						try
						{
							new_alpha =  100 - Double.parseDouble(m_txtAlpha.getText());
							
						} catch (Exception ex)
						{
						}
						
						m_sldAlpha.setValue((int)(100-new_alpha));
						m_txtAlpha.setText(String.format(Locale.US, "%.4f", 100-new_alpha));
						m_alpha = new_alpha;
					}
				}
			});
		
		m_sldSampleSize.addChangeListener(
			new ChangeListener()
			{
				public void stateChanged(ChangeEvent ev)
				{
					m_txtSampleSize.setText(String.valueOf(m_sldSampleSize.getValue()));
				}
			});

		m_sldSampleCount.addChangeListener(
			new ChangeListener()
			{
				public void stateChanged(ChangeEvent ev)
				{
					m_txtSampleCount.setText(String.valueOf(m_sldSampleCount.getValue()));
				}
			});

		m_sldTheta.addChangeListener(
				new ChangeListener()
				{
					public void stateChanged(ChangeEvent ev)
					{
						m_txtTheta.setText(String.valueOf(m_sldTheta.getValue() / 10.));
						m_view.setTheta(m_sldTheta.getValue() / 10.);
					}
				});
		
		m_sldAlpha.setValue(95);
		m_sldSampleSize.setValue(10);
		m_sldSampleCount.setValue(500);
		m_sldTheta.setValue(50);
		
		m_pnlMain.getButton("btnSample").addActionListener(
			new ActionListener()
			{
				public void actionPerformed(ActionEvent ev)
				{
					btnSample_actionPerformed(ev);
				}
			});
		
		m_lblRatio    = m_pnlMain.getFormAccessor("grdSettings").getLabel("lblRatio");
		m_lblEquation = m_pnlMain.getFormAccessor("grdSettings").getLabel("lblEquation");
		m_lblEquation.setIcon(new ImageIcon(createImage(ResRoot.class, "eqn.gif")));
	}
	
	
	private void btnSample_actionPerformed(ActionEvent ev)
	{
		int good_ci = m_view.sample(m_sldSampleSize.getValue(), m_sldSampleCount.getValue(), m_alpha/100.);
		
		m_lblRatio.setText(String.format("%d CIs contain theta (%.2f%%)", good_ci, (float)(good_ci*100./m_sldSampleCount.getValue())));
	}


	public static void main(String[] args)
	{
		try
		{
			UIManager.setLookAndFeel(new PlasticXPLookAndFeel());
			
		} catch (UnsupportedLookAndFeelException ex)
		{
		}
		
		MainFrame f = new MainFrame();
		f.setVisible(true);
	}

	public static Image createImage(Class clazz, String path) 
	{
		java.net.URL imgURL = clazz.getResource(path);
		if (imgURL != null) 
		{
			return Toolkit.getDefaultToolkit().createImage(imgURL);
			
		} else 
		{
			System.err.println("Couldn't find file: " + path);
			return null;
		}
        
        
	}
	
}
